/*
 * Astri_Monitor.h
 *
 *  Created on: Jul 3, 2021
 *      Author: astrisw
 */

#pragma once

#include <Base_Monitor.h>
#include <Base_Configurator.h>
#include <chrono>
#include <unordered_map>

namespace inaf::oasbo::PacketMonitors {
class AstriPacketMonitor: public BasePacketMonitor {

protected:
	AstriPacketMonitor();
	std::chrono::duration<double, std::milli> timeElps;
	std::chrono::steady_clock::time_point startTime;
	std::unordered_map<std::string, size_t> lastSSCMap;
	std::unordered_map<std::string, size_t> lostsMap;
	std::chrono::steady_clock::time_point lastNsTime;
	double bitsLastNs;
	size_t packsLastNs;
	uint update_interval;

	void monitPacketLost(PacketLib::BasePacket &pack);

	static std::string tot_packets_key;
	static std::string tot_MB_key;
	static std::string tot_lost_key;
	static std::string byte_rate_key;
	static std::string mbit_rate_key;
	static std::string packet_rate_key;
	friend class AstriPacketMonitorBuilder;

public:
	void setUpdateInterval(int interval);
	uint getUpdateInterval(){return update_interval;}
	void monit(PacketLib::BasePacket &pack);
	void reset();
	void printStats();
};



class AstriPacketMonitorBuilder {
protected:
	AstriPacketMonitor *monitor;

public:
	std::string config_target { "astrisimplemonitor" };
	std::string interval_key { "rateinterval" };

	AstriPacketMonitorBuilder();
	AstriPacketMonitorBuilder(std::string ip, int port);
	~AstriPacketMonitorBuilder();
	void reset();
	AstriPacketMonitorBuilder* configFrom(Configurators::BaseConfigurator &conf);
	AstriPacketMonitorBuilder* setUpdateInterval(int interval);
	AstriPacketMonitor* getMonitor();
};
}
